/*
   Copyright 2005 Logical Practice Systems Limited

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
/*
	AltHttpRequest.js
	This file depends on Debug.js and Base64.js being included before
	this one.
*/

// global configuration var
// must be set before you construct an AltHttpRequest object
var altHttpRequestUseCurl = false ;

/*
	Wrapper for either XMLHttpRequest or /usr/bin/curl.
	Note this is really meant for widgets and won't work
	in at least one other well know web browser.
*/
function AltHttpRequest(){
	this.onfinished ;
	this.status ;
	this.statusText ;
	this.responseText ;
	
	this.open = AltHttpRequest_open ;
	this.send = AltHttpRequest_send ;

	this.method ;
	this.url ;
	this.async ;
	this.username ;
	this.password ;

	this.httpbackend ;
	if(window.widget && altHttpRequestUseCurl == true ){
		this.httpbackend = new CurlHttpRequestBackEnd() ;
	}else{
		this.httpbackend = new XmlHttpRequestBackEnd();
	}
	this.httpbackend.parent = this ;
	this.responseComplete = AltHttpRequest_responseComplete ;
}


function AltHttpRequest_open( method , url , async , username , password){
	DEBUG('AltHttpRequest_open method='+method + ' url='+url+' async='+async+' username='+username+ ' password='+password);
	this.method = method ;
	this.url = url ;
	this.async = async ;
	this.username = username ;
	this.password = password ;
	return true ;
}

function AltHttpRequest_send(content){
	this.httpbackend.performRequest(this.method,this.url, this.async,this.responseComplete,this.username,this.password);
}

function AltHttpRequest_responseComplete( e ){
	DEBUG('responseComplete' + e );
	this.status = e.status ;
	this.statusText = e.statusText ;
	this.responseText = e.responseText ;
	
	this.onfinished(e);
}

var xmlhttp_currentRequest = null ;

function XmlHttpRequestBackEnd(){
	this.parent ;
	this.req ;
	this.performRequest = XmlHttpRequestBackEnd_performRequest ;
}

function XmlHttpRequestBackEnd_performRequest( method , url , async , callback , username , password ){
	DEBUG('XmlHttpRequestBackEnd_performRequest method='+method + ' url='+url+' async='+async+' username='+username+ ' password='+password);
	if( xmlhttp_currentRequest == null ){
		xmlhttp_currentRequest = this ;
		this.req = new XMLHttpRequest() ;
		this.req.onreadystatechange = XmlHttpRequestBackEnd_readyStateChange ;
		this.req.open(method,url,async);
		var authHeader ;
		authHeader = "Basic " + encode64(username+':'+password) ;
//		DEBUG(" authHeader = " + authHeader );
		this.req.setRequestHeader("Authorization", authHeader ) ;
		this.req.send();		
	}else{
		throw "xmlhttp_currentRequest is not null, must be a command in flight";
	}
}

function XmlHttpRequestBackEnd_readyStateChange(){
	var currentRequest = xmlhttp_currentRequest ;
	var req = currentRequest.req ;
	DEBUG('state=' + currentRequest.req.readyState);
	if( req.readyState == 4 ){
		DEBUG("readyStateChanged status=" + req.status + " text=" + req.statusText + " responsetext="+req.responseText);
//		DEBUG('readyStateChanged status=' + currentRequest.req.status + ' responseText=' + currentRequest.req.responseText);
		xmlhttp_currentRequest = null ;
		currentRequest.parent.responseComplete( {status:currentRequest.req.status,statusText:currentRequest.req.statusText,responseText:currentRequest.req.responseText});
	}
}

function CurlHttpRequestBackEnd() {
	this.parent ;
	this.command ;
	this.performRequest = CurlHttpRequestBackEnd_performRequest ;
	this.endHandler = CurlHttpRequestBackEnd_endHandler ;
	this.responseComplete ;
}

// grrr... I wish there was a better way.
var curl_currentRequest = null ;

function CurlHttpRequestBackEnd_performRequest( method , url , async , callback , username , password ){
	DEBUG('CurlHttpRequestBackEnd_performRequest method='+method + ' url='+url+' async='+async+' username='+username+ ' password='+password);
	if( curl_currentRequest == null ){
		curl_currentRequest = this ;
		if( window.widget ){
			this.command = widget.system('/usr/bin/curl --silent --get -u '+username+':'+password+' ' + url , this.endHandler);
			this.command.curlHttpBackEnd = this ;
		}
	}else{
		throw "curl_currentRequest is not null, must be a command in flight";
	}
}

function CurlHttpRequestBackEnd_endHandler( command ){
	DEBUG('CurlHttpRequestBackEnd_endHandler status=' + command.status + ' errorString=' + command.errorString );
	var parent = curl_currentRequest.parent ;
	curl_currentRequest = null ;
	
	if( command.status == 0 ){
		DEBUG('callback = ' + this.curlHttpBackEnd );
		parent.responseComplete( {status:200,statusText:'OK',responseText:command.outputString});		
	}else{
		parent.responseComplete( {status:500,statusText:'FAILED :'+command.errorString,responseText:null});
	}
}

